from UnitTesterSG.UnitTesterSGFunctions import *

def pytestDriver(failWithError=False):
    import UnitTesterSG.pytestDriver
    pytestDriver.runAllTests(failWithError=failWithError)

def runUnitTests():
    import UnitTesterSG.UnitTesterSGFunctions
    UnitTesterSG.UnitTesterSGFunctions.runAllTests()

def main():
    import UnitTesterSG.UnitTesterSGFunctions
    UnitTesterSG.UnitTesterSGFunctions.runAllTests()

if __name__=="__main__":
    import UnitTesterSG.UnitTesterSGFunctions
    UnitTesterSG.UnitTesterSGFunctions.runAllTests()


# -*- coding: utf-8 -*-
"""
Created on Tue Jun 12 14:07:44 2018

@author: Alex
"""
import sys
import os
import pathlib
pathToThisFile = pathlib.Path(__file__).parent.resolve() 
pathToLib = os.path.join(pathToThisFile, pathlib.Path("lib")) #using pathlib will work on any OS.
sys.path.insert(1, pathToLib) #If we just just used str(pathToThisFile) + ".\\lib" , the test would work locally, but online unit tests using pytest on Travis CI will fail, since that is linux based.
print("Inside test_11.py", sys.path)
pathToParent = os.path.join(pathToThisFile, pathlib.Path("..")) #using pathlib will work on any OS.
#import the functions from UnitTesterSG
import UnitTesterSG as ut
#import your function
from sumOfValues import sum_of_elements

#get the suffix argument for check_results
suffix = ut.returnDigitFromFilename(__file__)
#prefix. Make this '' if you do not want any prefix.
prefix = ''

#input for the unit that will be tested
inputArray = [1,2,3]

#These two lines can hardcode the expected results. They are not required. 
#expected_results = 6
#ut.set_expected_result(expected_results, str(expected_results), prefix = '', suffix=suffix)

#outputs with the function being tested using the input
output = sum_of_elements(inputArray)

#places the object in a tuple
resultObj = (output)

#String must be provided provided. Make it '' if you do not want to use a result string.
resultStr = str(resultObj)

#this is so that pytest can do UnitTesterSG tests.
def test_pytest(): #note that it cannot have any required arguments for pytest to use it, and that it is using variables that are defined above in the module.
    ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = False)
    
    
if __name__ == "__main__":
   #This is the normal way of using the UnitTesterSG module, and will be run by UnitTesterSG or by running this test file by itself.
   ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = True, interactiveTesting=True)



# -*- coding: utf-8 -*-
import collections


#This makes the sum of a Nested array/list/tuple.
#For example: [-1, 2, [3, 4, 5], 6] will give 19.
# and [1, 2, [3, 4, 5], 6] will give 21.
def sumNested(arr):
    currentSum = 0
    if isinstance(arr,collections.abc.Iterable):
        for elem in arr:
            returnSum = sumNested(elem)
            currentSum = currentSum + returnSum
        return currentSum
    else:  #if the code has gotten here, the input is a number and the "sum" is just itself.
        sumArr = arr
        return sumArr

sum_2dNested = sumNested #This really should have been the name originally.
    
#This makes the sum of absolute values of a Nested array/list/tuple.
#For example: [-1, 2, [3, 4, 5], 6] will give 21.
# and [1, 2, [3, 4, 5], 6] will give 21.
def sumNestedAbsValues(arrayOrNumber):
    currentSum = 0
    if isinstance(arrayOrNumber,collections.abc.Iterable):
        for elem in arrayOrNumber:
            returnSum = sumNestedAbsValues(elem)
            currentSum = currentSum + returnSum
        return currentSum
    else: #if the code has gotten here, the input is an indivudal number so here is where we take the absolute value.
        AbsVal = abs(arrayOrNumber)
        return AbsVal

sumNestedAbsValues_2dNested = sumNestedAbsValues #This really should have been the name originally.

#isNestedOrString takes an array and checks to see if it is iterable.  If it is iterable then it loops through the array
#to see if it has any more iterable objects.  If there are no iterable values in the array, then the array is not nested.
#Otherwise it has nesting.
#Examples: 3 is not iterable and will return false
#[1,2] is iterable but neither 1 nor 2 are iterable so isNestedOrString will return false
#[1,[2,3]] is iterable, 1 is not iterable but [2,3] is so isNestedOrString will return true
def isNestedOrString(arr):
    if isinstance(arr,collections.abc.Iterable):
        for elem in arr:
            if isinstance(elem,collections.abc.Iterable):
                return True
        #If it finishes the loop then it hasn't found a non-iterable object and is not nested
        return False
    #If the object is not iterable then it can't be nested
    else:
        return False

'''
This function has an implied return by modifying the array called subtractionResult
Typically subtractionResult would be a deepcopy of one of the two arrays
If there is a tuple or any other immutable type, the subtractNested function will not work
subtractionResult = copy.deepcopy(arr1)
subtractNested(arr1,arr2,subtractionResult)
#we do allow approximate comparisons using the variables relativeTolerance and absoluteTolerance
'''
def subtractNested(arr1,arr2,subtractionResult, relativeTolerance=None, absoluteTolerance=None, softStringCompare=False):
    if isinstance(arr1,collections.abc.Iterable):
        for elemindex,elem in enumerate(arr1):
            if type(elem) == str:
                if softStringCompare == True: #if using softStringCompare
                    #The 0 or 1 returned is opposite what is normally done in Python
                    #1 is usually true and 0 is usually false 
                    if stringCompare(arr1[elemindex],arr2[elemindex]): #Determine if the strings are equal using stringCompare
                        subtractionResult[elemindex] = 0
                    else:
                        subtractionResult[elemindex] = 1
                else: #otherwise compare regularly
                    #The 0 or 1 returned is opposite what is normally done in Python
                    #1 is usually true and 0 is usually false 
                    if arr1[elemindex] == arr2[elemindex]:
                        subtractionResult[elemindex] = 0
                    else:
                        subtractionResult[elemindex] = 1
            else: 
                if isinstance(elem,collections.abc.Iterable):
                    subtractNested(arr1[elemindex],arr2[elemindex],subtractionResult[elemindex], relativeTolerance=relativeTolerance, absoluteTolerance=absoluteTolerance, softStringCompare=softStringCompare)
                else: #this is for final elements, like integers and floats.
                    #we do allow approximate comparisons using the variables relativeTolerance and absoluteTolerance
                    # there are a variety of comparison tools, https://docs.pytest.org/en/documentation-restructure/how-to/builtin.html#comparing-floating-point-numbers
                    # we are using numpy allclose because we want to have  as few dependencies as possible, and numpy is not such a bad dependency in our view.
                    if (relativeTolerance==None and absoluteTolerance==None):
                        subtractionResult[elemindex] = arr1[elemindex] - arr2[elemindex]
                    else: #Else one of the tolerances requested is not None
                        import numpy as np 
                        #we 1st need to make any tolerances that are still none into the numpy default, because we don't know if the person has selected both tolerances.
                        #and we cannot feed "None" into numpy.
                        if relativeTolerance == None:
                            relativeTolerance = 1.0E-5
                            print("Warning: Can't have absolute tolerance without relative tolerance. Setting relative tolerance to 1.0E-5.")
                        if absoluteTolerance == None:
                            absoluteTolerance = 1.0E-8
                            print("Warning: Can't have relative tolerance without absolute tolerance. Setting absolute tolerance to 1.0E-8.")
                        #now we do the comparison
                        trueIfTheyAreApproximatelyEqual = np.allclose(arr1[elemindex],arr2[elemindex], rtol = relativeTolerance, atol = absoluteTolerance)
                        if trueIfTheyAreApproximatelyEqual == True:
                            subtractionResult[elemindex] = 0
                        if trueIfTheyAreApproximatelyEqual == False: #If they are not equal, we return the actual subtraction result.
                            subtractionResult[elemindex] = arr1[elemindex] - arr2[elemindex]
        #There is an implied return of arr3 since arr3 was overwritten in the function
    
                        

#This function converts a nested Structure into a nested list.
#For example, 
# nestedTuple = (1,2,(3,4,(5)),6)
# nestedList = nested_iter_to_nested_list(nestedTuple)
# yields: [1, 2, [3, 4, 5], 6]
def nested_iter_to_nested_list(iterReceived):
    #The first two lines are justs to return the object immediately if it's not an iterable.
    #This is mostly to prevent bugs if someone tries to feed an integer, for example.
    if not isinstance(iterReceived,collections.abc.Iterable):
        return iterReceived
    list_at_this_level = list(iterReceived)
    for elemIndex, elem in enumerate(iterReceived):
        #A string is iterable and a single value in a string is also iterable
        #So check to see if it is not a string to avoid recursion error
        if isinstance(elem,collections.abc.Iterable) and type(elem) != str:
            list_at_this_level[elemIndex] = nested_iter_to_nested_list(elem)
        else:
            list_at_this_level[elemIndex] = elem
    return list_at_this_level

'''
stringCompare takes in two strings and compares a standardized version of the two to see if they match
Added: 181008
Last modified: 181008
'''
def stringCompare(firstString,secondString):
    import re
    #First store the strings into a variable that will be standardized
    standardizedFirstString = firstString
    standardizedSecondString = secondString
    #Strip the strings of any whitespace on the outsides
    standardizedFirstString = standardizedFirstString.strip()
    standardizedSecondString = standardizedSecondString.strip()
    #Make both strings lowercase
    standardizedFirstString = standardizedFirstString.lower()
    standardizedSecondString = standardizedSecondString.lower()
    #Using regex, find any style of whitespace on the inside and replace it with a standardized space
    standardizedFirstString = re.sub('\s+',' ',standardizedFirstString)
    standardizedSecondString = re.sub('\s+',' ',standardizedSecondString)
    
    #If the standardized strings match return True
    if standardizedFirstString == standardizedSecondString:
        return True
    else: #Otherwise return false
        return False

#This is designed for arrays of numbers.
def makeAtLeast_2dNested(arr):
    import numpy as np
    if (type(arr) == type('str')) or ('int' in str(type(arr))) or ('float' in str(type(arr))): #If it's a string, int, or float, it's not nested.
        nestedArray = [[arr]]
    elif type(arr) != type('str'):#in the normal case, check if it's nested.
        if isNestedOrString(arr) == False:
            nestedArray = [arr]
        else: #Else it is already nested. However, if it is a list containing zero length numpy arrays, then it will become collapsed unless we make the numpy arrays atleast_1d.
            if type(arr) == type([]) and type(arr[0]) == type(np.array(0)):
                import copy
                nestedArray = copy.deepcopy(arr) #first make a copy, then change what is inside.
                for elementIndex in range(len(arr)):
                    nestedArray[elementIndex] = np.atleast_1d(nestedArray[elementIndex])
            else: #Else is the normal case.
                nestedArray = arr
    nestedArray=np.array(nestedArray)
    return nestedArray
    

def convertInternalToNumpyArray_2dNested(inputArray): #This is **specifically** for a nested array of the form [ [1],[2,3] ] Such that it is a 1D array of arrays.
    import numpy as np
    inputArray = np.array(inputArray)
    for elementIndex in range(0,len(inputArray)):
        inputArray[elementIndex] = np.array(inputArray[elementIndex])
    return np.array(inputArray)    

#Takes objects like this: [1,2, [3,4]] and returns objects like this: [1,2,3,4]
def flatten_2dNested(arr):
    import numpy as np
    arr = np.atleast_1d(arr)
    for elemIndex, elem  in enumerate(arr):
        if elemIndex == 0:
            flattenedElem = np.array(elem).flatten()
            flattenedArray = flattenedElem
        elif elemIndex >0:
            flattenedElem = np.array(elem).flatten()
            flattenedArray = np.hstack((flattenedArray, flattenedElem))
    return flattenedArray
    
#should return true for  [[1],[1,2]] and return false for [[1,2],[1,2]]
def checkIfStaggered_2dNested(arr):
        try:
            arr = np.array(convertInternalToNumpyArray(arr))
            onesArray = np.ones(np.shape(arr))
            onesArray = np.matmul(arr.transpose(), onesArray) #This is the key line and catches staggered arrays.
            return False
        except:
            return True
            
#Takes an arrayLike object like [1,2,3,4] and returns a nested version like [[1],[2],[3],[4]]            
def convertToNested(inputList):
    nestedList = []
    for individualItem in inputList:
        nestedList.append([individualItem])
    return nestedList


# -*- coding: utf-8 -*-
"""
Created on Tue Jun 12 14:07:44 2018

@author: Alex
"""
import sys
sys.path.insert(1, "..\\lib")
sys.path.insert(1, "..")
#import the functions from UnitTesterSG
import UnitTesterSG as ut
    
#get the suffix argument for check_results
suffix = ut.returnDigitFromFilename(__file__)
#prefix. Make this '' if you do not want any prefix.
prefix = ''

resultObj = (1,2,3)

#String must be provided provided. Make it '' if you do not want to use a result string.
resultStr = str(resultObj)

#this is so that pytest can do UnitTesterSG tests.
def test_pytest(): #note that it cannot have any required arguments for pytest to use it, and that it is using variables that are defined above in the module.
    ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = False)
    
    
if __name__ == "__main__":
   #This is the normal way of using the UnitTesterSG module, and will be run by UnitTesterSG or by running this test file by itself.
   ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = True, interactiveTesting=True)



# -*- coding: utf-8 -*-
"""
There are two ways to use UnitTesterSG: either with a known expected response, or simply comparing to a stored response.
This file is an example and template for a case with a known (e.g., analytical or otherwise calculated) expected response.
Just name your file test_N where N is an integer.

#NOTE: FOR THE TEMPLATE AS DISTRIBUTED: WHEN YOU RUN THIS FILE, IT WILL SAY THE **EXPECTED RESULT** MATCHES BUT THAT THE **EXPECTED RESULT STRING** DOES NOT MATCH. IT IS PERFECTLY FINE THAT THE RESULT STRING DOES NOT MATCH. THAT IS A TYPICAL SITUATION AND A FEATURE. IT DOES NOT MEAN THE TEST FAILED.
"""

#These "sys" lines are mainly because this are standard lines in our examples. Normally, you would not include these three lines.
import sys
sys.path.insert(1, ".\\lib")
sys.path.insert(1, "..")



#import the functions from UnitTesterSG
import UnitTesterSG as ut

#The below lines are typical code. There is no need to modify them.
#get the suffix argument for check_results
suffix = ut.returnDigitFromFilename(__file__)
#prefix. Make this '' if you do not want any prefix.
prefix = ''


"""
#This file is an example/template for when we ***know*** what result to expect.
#In the original template example, when we put in the number 4, we expect to get two arrays: One with values  [3,4,5]  and another with [32,64] Note that they are of different lengths. This is not a problem for UnitTesterSG.
#We must make a single results object: we will put lists with those values inside. Recognize that we are expecting array objects, but we can define our analytical result as a list. This is part of why UnitTesterSG was developed, since it compares what is inside, not just the objects.
"""
#input for the unit that will be tested
inputValue = 4
expectedFirstPart = [3,4,5]
expectedSecondPart = [32,64]
expectedResult = (expectedFirstPart,expectedSecondPart) #We are using a tuple, but it this could have been a list.

ut.set_expected_result(expectedResult,expected_result_str=str(expectedResult), prefix=prefix,suffix=suffix) #This is the typical syntax if you want to force an analytical result for your test.


"""
#Calculate our function outputs (actual results). We can functions from another module in this section.
"""
import numpy as np
def calculationFirstPart(inputNumber):
    if inputNumber == 4:
        return 3.0000001+np.array([0,1,2]) #This will return an array similar to [3,4,5]. Believe me. We are intentionally using non integer values to demonstrate the tolerances feature.

def calculationSecondPart(inputNumber):
    if inputNumber == 4:
        return 32.00000001*np.array([1.,2.]) #This will return an array similar to [32,64]. Believe me. We are intentionally using non integer values to demonstrate the tolerances feature.

#inputValue = 4 #This input is the same as above. Just reproduced for clarity. 
#outputs with the function being tested using the input
outputFirstPart = calculationFirstPart(inputValue)
outputSecondPart = calculationSecondPart(inputValue)
actualResult = (outputFirstPart, outputSecondPart)

"""We put our actual result into the resultObj variable."""
#put this in the resultObject
resultObj = actualResult

#String must be provided provided. Make it '' if you do not want to use a result string.
resultStr = str(resultObj)


"""We set our tolerances. There can be some rounding when the tolerances get checked, so they are not exact."""
relativeTolerance = 1.0E-5
absoluteTolerance = 1.0E-8


#this is so that pytest can do UnitTesterSG tests.
def test_pytest(): #note that it cannot have any required arguments for pytest to use it, and that it is using variables that are defined above in the module.
    ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = False, relativeTolerance=relativeTolerance, absoluteTolerance=absoluteTolerance)

"""#For any individual test, after finishing getting it working, set allowOverwrite to False in the line below calling doTest if you want to skip UnitTesterSG from stopping to notify user when results match but result strings don't. """        
if __name__ == "__main__":
   #This is the normal way of using the UnitTesterSG module, and will be run by UnitTesterSG or by running this test file by itself.
   ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = True, relativeTolerance=relativeTolerance, absoluteTolerance=absoluteTolerance, interactiveTesting = True)



# -*- coding: utf-8 -*-
"""
There are two ways to use UnitTesterSG: either with a known expected response, or simply comparing to a stored response.
This file is an example and template for a case where we can't simply 'provide' a solution so must compare to an **existing from before** output.

TO USE THIS TEMPLATE/EXAMPLE, YOU MUST RUN THIS FILE ONE TIME TO INITIATE. THE FIRST TIME THIS FILE IS RUN, IT WILL SAY THAT THE CALCULATED RESULT AND EXPECTED RESULT DO NOT MATCH -- THAT IS BECAUSE THE EXPECTED RESULT DOES NOT YET EXIST. DURING RUNTIME, THE SECOND QUESTION THE PROGRAM ASKS WILL BE IF YOU WANT TO OVERRWRITE OR CREATE EXPECTED RESULTS FILES: CHOOSE "Y"  AFTER DOING SO, THE EXPECTED RESULT HAS BEEN SET.  RUN THIS FILE AGAIN. SINCE YOU ARE RUNNING ON THE SAME COMPUTER, THE RESULT WILL BE THE SAME AND THE TEST WILL PASS. THE RESULT HAS BEEN STORED, SO THE TEST DIRECTORY CAN NOW BE INCLUDED IN A REPOSITORY OR COPIED TO ANOTHER COMPUTER FOR TESTING.

IF YOU WISH TO RESET THE STORED EXPECTED RESULTS TO NOT BEING SET YET, DELETE THE FILES THAT BEGIN WITH THE WORD EXPECTED.

You may copy this file and modify it to make your own test. Just name your file test_N where N is an integer.
"""

#These "sys" lines are mainly because this are standard lines in our examples. Normally, you would not include these three lines.
import sys
sys.path.insert(1, ".\\lib")
sys.path.insert(1, "..")



#import the functions from UnitTesterSG
import UnitTesterSG as ut

#The below lines are typical code. There is no need to modify them.
#get the suffix argument for check_results
suffix = ut.returnDigitFromFilename(__file__)
#prefix. Make this '' if you do not want any prefix.
prefix = ''


"""
#This file is an example/template for when we ***don't have an analytical result*** but we know our function is working.
#We know the function is working during template distribution because we are just using the test 12 example.
In this template, we ***will not*** use the "set_expected_result" command. So we are commenting out the below lines, and will go directly to using the function to create an actual output.
"""
# # # #input for the unit that will be tested
# # # inputValue = 4
# # # expectedFirstPart = [3,4,5]
# # # expectedSecondPart = [32,64]
# # # expectedResult = (expectedFirstPart,expectedSecondPart) #We are using a tuple, but it this could have been a list.

# # # ut.set_expected_result(expectedResult,expected_result_str=str(expectedResult), prefix=prefix,suffix=suffix) #This is the typical syntax if you want to force an analytical result for your test.


"""
#Calculate our function outputs (actual results). We can functions from another module in this section.
"""
import numpy as np
def calculationFirstPart(inputNumber):
    if inputNumber == 4:
        return 3.0000001+np.array([0,1,2]) #This will return an array similar to [3,4,5]. Believe me. We are intentionally using non integer values to demonstrate the tolerances feature.

def calculationSecondPart(inputNumber):
    if inputNumber == 4:
        return 32.00000001*np.array([1.,2.]) #This will return an array similar to [32,64]. Believe me. We are intentionally using non integer values to demonstrate the tolerances feature.

inputValue = 4 #This input is the same as above. Just reproduced for clarity. 
#outputs with the function being tested using the input
outputFirstPart = calculationFirstPart(inputValue)
outputSecondPart = calculationSecondPart(inputValue)
actualResult = (outputFirstPart, outputSecondPart)

"""We put our actual result into the resultObj variable."""
#put this in the resultObject
resultObj = actualResult

#String must be provided provided. Make it '' if you do not want to use a result string.
resultStr = str(resultObj)


"""We set our tolerances. There can be some rounding when the tolerances get checked, so they are not exact."""
relativeTolerance = 1.0E-5
absoluteTolerance = 1.0E-8


#this is so that pytest can do UnitTesterSG tests.
def test_pytest(): #note that it cannot have any required arguments for pytest to use it, and that it is using variables that are defined above in the module.
    ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = False, relativeTolerance=relativeTolerance, absoluteTolerance=absoluteTolerance)
    
"""#For any individual test, after finishing getting it working, set allowOverwrite to False in the line below calling doTest if you want to skip UnitTesterSG from stopping to notify user when results match but result strings don't. """        
if __name__ == "__main__":
   #This is the normal way of using the UnitTesterSG module, and will be run by UnitTesterSG or by running this test file by itself.
   ut.doTest(resultObj, resultStr, prefix=prefix,suffix=suffix, allowOverwrite = True, relativeTolerance=relativeTolerance, absoluteTolerance=absoluteTolerance, interactiveTesting=True)



import os
import sys

def runAllTests(failWithError=False):
    #in Python, the listdir command  returns files and directories (like typeing in "dir").
    listOfDirectoriesAndFiles = os.listdir(".")

    #Below is going to become a list of directories only in the next loop.
    directoryList = []
    for elem in listOfDirectoriesAndFiles:
        if os.path.isdir(elem) == True:
            directoryList.append(elem)
 
    allTestsPassed = True #initializing a flag to check if all tests have passed. Will change to False if any tests fail.
    #This loop goes into each directories, runs the specified command, and comes back.
    for directory in directoryList:
        print("Changing directory to "+directory)
        os.chdir(directory)
        
        try:
            os.system("del __pycache__ /Q") #for windows
            os.system("rm -r __pycache__ /Q") #for linux
        except:
            pass
        
        #We will check if __init__ exists, because for various situations, pytest.main( will have failures if there is no __init__.
        initExists = os.path.exists('__init__.py')
        if initExists:        
            #Try to run the test. In the past, we we used an executable version of pytest, but now we are using "pytest.main()" so we can get the exit code, that way we can fail with error if a unit test doesn't pass.
            import pytest
            exitCode = pytest.main()
            # https://stackoverflow.com/questions/8338854/how-to-run-py-test-against-different-versions-of-python
            if exitCode >= 1 and exitCode <5:
                allTestsPassed = False
        else: #if __init__ does not exist, we can usually still run the unit tests by running the pytest executable.
            os.system(sys.executable +" -m pytest") #this is like typing "python -m pytest" but uses whichever version of python should be used, important for virtual environments and different systems
        os.chdir("..")
    
    if allTestsPassed == False:
        print("At least one unit test failed.")
        if failWithError == True: #if the failWithError flag is on due to the optional argument, we will check if all tests passed. If not, we'll raise an error.
            raise RuntimeError("At least one unit test failed.")  #This is to intentionally create an error so that the Travis CI will fail.


#!/usr/bin/env python
# -*- coding: utf-8 -*-

#This was modified from https://github.com/navdeep-G/setup.py/blob/master/setup.py
#Which is part of the repository https://github.com/navdeep-G/setup.py
#Which is part of the public domain. 
# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os
import sys
from shutil import rmtree
from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'UnitTesterSG'
DESCRIPTION = 'UnitTesterSG is for unit testing scientific/engineering outputs. It accommodates nested and even staggered array type structures, and mixed data types (like strings and integers etc). It also enables you to set numerical and absolute tolerances.'
URL = 'https://github.com/AdityaSavara/UnitTesterSG'
EMAIL = 'AditySavara2008@u.northwestern.edu'  
AUTHOR = 'Aditya Savara'
REQUIRES_PYTHON = '>=3.0.0'
VERSION = '6.1.0'
LICENSE = 'BSD-3-Clause'

# What packages are required for this module to be executed?
REQUIRED = [
    'numpy',
    # 'requests', 'maya', 'records', #numpy...
]

EXTRAS = {
    'COMPLETE': ['colorama', 'pytest'] #This is a list.
}

#To make sure the license etc. is included, I added the DATA_FILES object based on https://stackoverflow.com/questions/9977889/how-to-include-license-file-in-setup-py-script
DATA_FILES = [
       ("", ["./UnitTesterSG/LICENSE.txt", "./UnitTesterSG/MANUAL.txt"]),
       ]

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    data_files = DATA_FILES,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license=LICENSE,
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
    },
)
